package board;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;


public class BoardDAO {
	
	private final static int GROUP_SIZE = 3;
	private final static int PAGE_SIZE = 10;
	private int totalCount;
	private int pageCount;
	
	Connection conn = null;
	PreparedStatement pstmt = null;
	ResultSet rs = null;
	
	String jdbc_driver = "oracle.jdbc.driver.OracleDriver";
	String jdbc_url = "jdbc:oracle:thin:@localhost:1521";
	public void connect(){
		try{
			Class.forName(jdbc_driver);
			conn = DriverManager.getConnection(jdbc_url,"sampleboard","1234");
			System.out.println("DB connection successed");
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	public void disconnect(){
		if(rs != null){
			try{
				rs.close();
			}catch(SQLException e){
				e.printStackTrace();
			}
		}
		if(pstmt != null){
			try{
				pstmt.close();
			}catch(SQLException e){
				e.printStackTrace();
			}
		}
		if(conn != null){
			try{
				conn.close();
			}catch(SQLException e){
				e.printStackTrace();
			}
		}
	}
	
	//Խù 
	public boolean insertDB(BoardBean board){
		connect();
		String sql = "INSERT INTO SAMPLEBOARD(B_ID, B_NAME, B_PASSWD, B_TITLE, B_CONTENTS, B_DATE, B_MASTERNUM, B_COUNT, B_REPLYNUM, B_STEP) VALUES(SAMPLEBOARD_SEQ.NEXTVAL,?,?,?,?,SYSDATE,SAMPLEBOARD_SEQ.CURRVAL,0,0,0)";
		try{
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, board.getB_name());
			pstmt.setString(2, board.getB_passwd());
			pstmt.setString(3, board.getB_title());
			pstmt.setString(4, board.getB_contents());
			pstmt.executeUpdate();
		}catch(Exception e){
			e.printStackTrace();
			return false;
		}finally{
			disconnect();
		}
		return true;
	}
	
	//õ
	public boolean insertReply(BoardBean board){
		connect();
		//     ϱ  ü .
		String replynum_sql = "UPDATE SAMPLEBOARD SET B_REPLYNUM = B_REPLYNUM+1 WHERE B_ID = ? AND B_REPLYNUM > ?";
		//õ
		String rep_insert_sql="INSERT INTO SAMPLEBOARD(B_ID, B_NAME, B_PASSWD, B_TITLE, B_CONTENTS, B_DATE, B_MASTERNUM, B_COUNT, B_REPLYNUM, B_STEP) VALUES(SAMPLEBOARD_SEQ.NEXTVAL,?,?,?,?,SYSDATE,?,0,?,?)";
		try{
			pstmt = conn.prepareStatement(replynum_sql);
			pstmt.setInt(1, board.getB_id());
			pstmt.setInt(2, board.getB_replynum());
			pstmt.executeUpdate();
			//õ
			pstmt = conn.prepareStatement(rep_insert_sql);
			pstmt.setString(1, board.getB_name());
			pstmt.setString(2, board.getB_passwd());
			pstmt.setString(3, board.getB_title());
			pstmt.setString(4, board.getB_contents());
			pstmt.setInt(5, board.getB_masternum());
			pstmt.setInt(6, board.getB_replynum()+1);
			pstmt.setInt(7, board.getB_step()+1);
			pstmt.executeUpdate();
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			disconnect();
		}
		
		return true;
	}
	
	public ArrayList getDBList(int curPage){
		connect();
		int end = 1;
		ArrayList<BoardBean> datas = new ArrayList<BoardBean>();
		String count_sql = "select count(*) from SAMPLEBOARD";
		String sql="SELECT B_ID, B_NAME, B_TITLE, B_DATE, B_COUNT, B_STEP FROM SAMPLEBOARD ORDER BY B_MASTERNUM DESC, B_REPLYNUM, B_STEP, B_ID";

		try{
			setCount(count_sql);
			pstmt = conn.prepareStatement(sql,ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
			ResultSet rs = pstmt.executeQuery();
			
			if(curPage > 1){
				rs.absolute((curPage-1)*PAGE_SIZE);
			}
			
			while(rs.next() && (end <=PAGE_SIZE)) {
				BoardBean board = new BoardBean();
				board.setB_id(rs.getInt("b_id"));
				board.setB_name(rs.getString("b_name"));
				board.setB_title(rs.getString("b_title"));
				board.setB_date(rs.getDate("b_date"));
				board.setB_count(rs.getInt("b_count"));
				board.setB_step(rs.getInt("b_step"));
				datas.add(board);
				end++;
			}
			rs.close();
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			disconnect();
		}
		return datas;
	}
	
	public BoardBean getDB(int b_id){
		connect();
		BoardBean board = new BoardBean();
		
		String sql = "SELECT B_NAME, B_TITLE, B_CONTENTS, B_STEP FROM SAMPLEBOARD WHERE B_ID = ?";
		try{
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, b_id);
			rs = pstmt.executeQuery();
			if(rs.next()){
				board.setB_id(b_id);
				board.setB_name(rs.getString("b_name"));
				board.setB_title(rs.getString("b_title"));
				board.setB_contents(rs.getString("b_contents"));
				board.setB_step(rs.getInt("b_step"));
			}
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			disconnect();
		}
		return board;
	}
	public BoardBean getDB_Reply(int b_id){
		connect();
		BoardBean board = new BoardBean();
		
		String sql = "SELECT B_NAME, B_TITLE, B_CONTENTS, B_STEP, B_MASTERNUM, B_REPLYNUM FROM SAMPLEBOARD WHERE B_ID = ?";
		try{
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, b_id);
			rs = pstmt.executeQuery();
			if(rs.next()){
				board.setB_id(-1);
				board.setB_name(rs.getString("b_name"));
				board.setB_title("RE:" + rs.getString("b_title"));
				board.setB_contents(getReplyContent(rs.getString("b_contents")));
				board.setB_step(rs.getInt("b_step"));
				board.setB_masternum(rs.getInt("b_masternum"));
				board.setB_replynum(rs.getInt("b_replynum"));
			}
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			disconnect();
		}
		return board;
	}
	
	public boolean deleteDB(int b_id, int b_step){
		connect();
		String sql="";
		if(b_step == 0){	//  õ Բ 
			sql = "DELETE FROM SAMPLEBOARD WHERE B_MASTERNUM = ?";
		}else{				//ϰ ø 
			sql = "DELETE FROM SAMPLEBOARD WHERE B_ID = ?";
		}
		try{
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, b_id);
			pstmt.executeUpdate();
		}catch(Exception e){
			e.printStackTrace();
			return false;
		}finally{
			disconnect();
		}
		
		return true;
	}
	
	public boolean updateDB(BoardBean board){
		connect();
		String sql="UPDATE SAMPLEBOARD SET B_NAME=?, B_PASSWD=?, B_TITLE=?, B_CONTENTS=?, B_DATE=SYSDATE WHERE B_ID =?";
		try{
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, board.getB_name());
			pstmt.setString(2, board.getB_passwd());
			pstmt.setString(3, board.getB_title());
			pstmt.setString(4, board.getB_contents());
			pstmt.setInt(5, board.getB_id());
			pstmt.executeUpdate();
		}catch(Exception e){
			e.printStackTrace();
			return false;
		}finally{
			disconnect();
		}
		return true;
	}
	
	public boolean checkPwd(int b_id, String pwd){
		connect();
		boolean result = false;
		String sql = "SELECT B_PASSWD FROM SAMPLEBOARD WHERE B_ID=?";
		try{
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, b_id);
			rs = pstmt.executeQuery();
			if(rs.next() && pwd.equals(rs.getString(1))){
				result=true;
			}else{
				result=false;
			}
		}catch(Exception e){
			e.printStackTrace();
		}finally{
			disconnect();
		}
		return result;
	}
	public String getPageNav(int curPage){
		StringBuffer retstr = new StringBuffer();
		
		// ׷ 
		int curGroup = (curPage-1) / GROUP_SIZE;
		int linkPage = curGroup * GROUP_SIZE;
		// ù° ׷ ƴѰ
		if(curGroup > 0) {
			retstr.append("[<a href=list.do?page=1>ó</a>]");
			retstr.append("[<a href=list.do?page=")
				  .append(linkPage)
				  .append("></a>]");
		}
		else {
			retstr.append("[ó] []");
		}
		//  ũ  Ŵ
		linkPage++;

		int loopCount = GROUP_SIZE;
		// ׷  ũ.
		while((loopCount > 0) && (linkPage <= pageCount)) {
			if(linkPage == curPage){
				retstr.append(linkPage);
			}
			else{
				retstr.append("[<a href=list.do?page=")
					  .append(linkPage)
					  .append(">")
					  .append(linkPage)
					  .append("</a>]&nbsp;");
			}
			linkPage++;
			loopCount--;
		}
		// ׷ ִ 
		if(linkPage <= pageCount) {
			retstr.append("[<a href=list.do?page=")
				  .append(linkPage)
				  .append("></a>]");
			retstr.append("[<a href=list.do?page=")
				  .append(pageCount)
				  .append("></a>]");
		}
		else {
			retstr.append("[] []");
		}
		return retstr.toString();
	}
	
	public void setCount(String sql)throws Exception{
		
		try{
			pstmt = conn.prepareStatement(sql);
			rs = pstmt.executeQuery();
			if(rs.next()){
				totalCount = rs.getInt(1);
			}
		}catch(SQLException ex){
			ex.printStackTrace();
		}
		pageCount = (int)Math.ceil(totalCount / (PAGE_SIZE+0.0));
		
	}
	
	
//	 亯ۿ >> ȣ ̴ ޼
	public String getReplyContent(String content) {
  		int pos=0;
		String rcontent = ">" + content;
		while((pos = rcontent.indexOf("\n",pos)) != -1) {
			String left = rcontent.substring(0,pos+1);
			String right = rcontent.substring(pos+1,rcontent.length());
			rcontent = left + ">"+right;
			pos +=2;
		}
		return rcontent;
	}
	
	public void countPlus(int b_id,int curCount) {
		connect();
		String sql = "UPDATE SAMPLEBOARD SET B_COUNT = ? WHERE B_ID = ?";
		
		try {
	 		pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1,curCount+1);
			pstmt.setInt(2,b_id);
			pstmt.executeUpdate();
		}catch(SQLException e) {
			e.printStackTrace();
		}finally{
			disconnect();
		}
	}
	
	

}
